/*
    This class implements rectangles with immutable perimeter.

    When a RestrictedRectangle object is created the
    constructor is passed two parameters, length and
    width. But the object instead has the two fields
    length and perimeter. The perimeter field is computed
    from the length and width constructor parameters.

    Once the perimeter is set, it can't be changed (it
    becomes "immutable"). The other "fields" (length and
    width) can only be changed relative to the restriction
    of keeping the perimeter constant (hence the name of
    the class, RestrictedRectangle).

    The public interface appears to have length and width
    fields, but the private implementation has length and
    perimeter fields. It is important to realize that the
    public get and set methods that a class provides don't
    need to be exactly like the actual fields hidden in the
    private implementation of the class.


    Use DrJava's interactions pane (the Java interpreter)
    to experiment with RestrictedRectangle objects.
*/

public class RestrictedRectangle
{
   private double perimeter;
   private double length;

   public RestrictedRectangle(double length, double width)
   {
      this.perimeter = 2*length + 2*width;
      this.length = length;
   }

   public void setLength(double length)
   {
      if (length > perimeter/2)
         throw new IllegalArgumentException("length exceeds half of perimeter");
      else
         this.length = length;
   }

   public void setWidth(double width)
   {
      if (width > perimeter/2)
         throw new IllegalArgumentException("width exceeds half of perimeter");
      else
         setLength( (perimeter - 2*width)/2 );
   }

   public double getLength(){ return length; }

   public double getWidth(){ return (perimeter - 2*length)/2; }

   public double getArea(){ return getWidth()*getLength(); }

   public double getPerimeter(){ return perimeter; }

   public String toString()
   {
      return "RestrictedRectangle: " + ",\n\t width = " + getWidth()
                                        + ", length = " + getLength();
   }

}//RestrictedRectangle
