/*
   This class defines TemperatureReading3 objects.

   The only difference between this class and the
   TemperatureReading2 class is that this class adds
   several static fields and two static methods. The
   static fields are usefull temperature constants, and
   the two static methods are for converting temperatures
   (that are not stored in an object) from one scale to
   another.
*/

public class TemperatureReading3
{
   // three instance variables
   private double degreesKelvin;
   private double latitude;
   private double longitude;

   // some static variables (these don't have to be
   // declared final, but it is a good idea to)
   public static final double BOILING_C = 100.0;
   public static final double BOILING_F = 212.0;
   public static final double FREEZING_C = 0.0;
   public static final double FREEZING_F = 32.0;

   // the default constructor
   public TemperatureReading3()
   {
      degreesKelvin = 0;
      latitude = 0;
      longitude = 0;
   }

   // another constructor
   public TemperatureReading3(double temperature, char scale,
                              double newLat, double newLong)
   {
      degreesKelvin = 0; //in case scale is neither C nor F
      if (scale == 'C' )
          degreesKelvin = temperature + 273;
      if (scale == 'F')
          degreesKelvin = (temperature - 32)/1.8 + 273;
      // now validate the data so that there cannot be an
      // unrealistic value for the temperature
      if ( degreesKelvin < 0 ) degreesKelvin = 0;

      // now initialize the other two fields
      latitude = newLat;
      longitude = newLong;
   }

   // two get methods (accessor methods) for temperature
   public double getTemperatureC()
   {
       return degreesKelvin - 273;
   }

   public double getTemperatureF()
   {
       return (degreesKelvin - 273)*1.8 + 32;
   }

   // get methods for latitude and longitude
   public double getLatitude()
   {
       return latitude;
   }

   public double getLongitude()
   {
       return longitude;
   }

   // two set methods (mutator methods) for temperature
   public void setTemperatureC(double degreesC)
   {
       if (degreesC >= -273)  //validate the data
           degreesKelvin = degreesC + 273;
       else
           degreesKelvin = 0;
   }

   public void setTemperatureF(double degreesF)
   {
       degreesKelvin = (degreesF - 32)/1.8 + 273;
       //now validate the data
       if ( degreesKelvin < 0 ) degreesKelvin = 0;
   }

   // set methods for latitude and longitude
   public void setLatitude(double newLat)
   {
       latitude = newLat;
   }

   public void setLongitude(double newLong)
   {
       longitude = newLong;
   }

   //two static methods
   public static double convertCtoF(double degreesC)
   {
      return (degreesC*1.8)+32;
   }

   public static double convertFtoC(double degreesF)
   {
       return (degreesF - 32)/1.8;
   }

}//TemperatureReading3