/*
   This class defines TemperatureReading4 objects.

   The main difference between this class and the
   TemperatureReading3 class is that this class adds
   a new member (field) to the objects. The new member
   is itself an object, an object of type Date. The
   Date class is a class that is predefined in the Java
   libraries.

   This class also has a toString method.

   This file also has Javadoc documentation in it for
   creating an HTML documentation file.
*/
import java.util.*;

/**
    This class encapsulates the data from a temperature reading.
    The details of how a temperature is stored are hidden from the
    user of objects created from this class. The public interface of
    this class requires that the scale of a temperature must always
    be specified. This provides a safe way to store temperature values.

    @author Roger Kraft
    @version 4
*/
public class TemperatureReading4
{
   // four instance variables (the last of which is
   // itself an object, a Date object)
   private double degreesKelvin;
   private double latitude;
   private double longitude;
   private Date   timeOfReading;

   // some static variables (these don't have to be
   // declared final, but it is a good idea to)
   public static final double BOILING_C = 100.0;
   public static final double BOILING_F = 212.0;
   public static final double FREEZING_C = 0.0;
   public static final double FREEZING_F = 32.0;

   /**
      The default constructor.
   */
   public TemperatureReading4()
   {
      degreesKelvin = 0;
      latitude = 0;
      longitude = 0;
      // Construct a new Date object. According to the
      // Java documentation, the default Date constructor
      // creates a Date object with the time of its creation.
      timeOfReading = new Date();
   }

   /**
      A constructor that specifies the scale for the temperature.

      @param temperature    the temperature in either Celsius or Fahrenheit
      @param scale          specifies the scale for temperature, either 'F' or 'C'
      @param newLat         the latitude of the reading
      @param newLong        the longitude of the reading
   */
   public TemperatureReading4(double temperature, char scale,
                              double newLat, double newLong)
   {
      degreesKelvin = 0; //in case scale is neither C nor F
      if (scale == 'C' )
          degreesKelvin = temperature + 273;
      if (scale == 'F')
          degreesKelvin = (temperature - 32)/1.8 + 273;
      // now validate the data so that there cannot be an
      // unrealistic value for the temperature
      if ( degreesKelvin < 0 ) degreesKelvin = 0;

      // now initialize the latitude and longitude fields
      latitude = newLat;
      longitude = newLong;
      // construct a new Date object
      timeOfReading = new Date();
   }

   /**
       Retrieve the temperature value using the Celsius scale.
   */
   public double getTemperatureC()
   {
       return degreesKelvin - 273;
   }

   /**
       Retrieve the temperature value using the Fahrenheit scale.
   */
   public double getTemperatureF()
   {
       return (degreesKelvin - 273)*1.8 + 32;
   }

   // get methods for latitude, longitude, and timeOfReading
   public double getLatitude()
   {
       return latitude;
   }

   public double getLongitude()
   {
       return longitude;
   }

   public Date getTimeOfReading()
   {
       return timeOfReading;
   }

   /**
       Set the temperature value using the Celsius scale. (If the temperature value given is not realistic, the temperature will be set to the default temperature.)

       @param degreesC     new temperature value in degreees Celsius
   */
   public void setTemperatureC(double degreesC)
   {
       if (degreesC >= -273)  //validate the data
           degreesKelvin = degreesC + 273;
       else
           degreesKelvin = 0;
   }

   /**
       Set the temperature value using the Fahrenheit scale. (If the temperature value given is not realistic, the temperature will be set to the default temperature.)

       @param degreesF     new temperature value in degreees Fahrenheit
   */
   public void setTemperatureF(double degreesF)
   {
       degreesKelvin = (degreesF - 32)/1.8 + 273;
       //now validate the data
       if ( degreesKelvin < 0 ) degreesKelvin = 0;
   }

   // set methods for latitude, longitude, and timeOfReading
   public void setLatitude(double newLat)
   {
       latitude = newLat;
   }

   public void setLongitude(double newLong)
   {
       longitude = newLong;
   }

   public void setTimeOfReading(Date newDate)
   {
       timeOfReading = newDate;
   }

   // a toString method
   public String toString()
   {
      return "The tempreature is " + getTemperatureC() + "C which is "
              + getTemperatureF() + "F,\n"
              + " the latitude is " + getLatitude()
              + " and the longitude is " + getLongitude() + ",\n"
              + " and the time of the reading is " + getTimeOfReading().toString();
   }

   /**
       A static method that converts numbers from a Celsius value to a Fahrenheit value.

       @param degreesC     a number that is interpreted as a temperature in degrees Celsius
   */
   public static double convertCtoF(double degreesC)
   {
      return (degreesC*1.8)+32;
   }

   /**
       A static method that converts numbers from a Fahrenheit value to a Celsius value.

       @param degreesF     a number that is interpreted as a temperature in degrees Fahrenheit
   */
   public static double convertFtoC(double degreesF)
   {
       return (degreesF - 32)/1.8;
   }

}//TemperatureReading4